<?php


namespace Schaffen\ProductionOrder\ProductionOrder\Managers;


use Freelabois\LaravelQuickstart\Extendables\ManipulationManager;
use Illuminate\Support\Facades\DB;
use Schaffen\Core\Product\Repositories\ProductVarietyPackRepository;
use Schaffen\ProductionOrder\ProductionOrder\Repositories\CompositeProductRepository;
use Schaffen\ProductionOrder\ProductionOrder\Repositories\ProductionOrderRepository;
use Schaffen\ProductionOrder\ProductionOrder\Repositories\CompositeProductHistoricRepository;
use Illuminate\Support\Carbon;

class CompositeProductManager extends ManipulationManager
{
    protected $product_variety_pack_repository;

    public function __construct(
        CompositeProductRepository $repository, 
        ProductVarietyPackRepository $product_variety_pack_repository, 
        ProductionOrderRepository $production_order_repository,
        CompositeProductHistoricRepository $composite_product_historic_repository
    ) 
    {
        parent::__construct($repository);
        $this->product_variety_pack_repository = $product_variety_pack_repository;
        $this->production_order_repository = $production_order_repository;
        $this->composite_product_historic_repository = $composite_product_historic_repository;

    }

    public function save($values, array $relations = [], $operation)
    {
        $final_product = $this->product_variety_pack_repository
            ->firstOrFail(["pve_codigo" => $values['pcp_fk_pve_codigo_acabado']])
            ->load(['composition', 'product_variety']);

        if (!$final_product) {
            throw new \Exception("O produto final selecionado não existe");
        }

        if (count($final_product->composition)) {
            throw new \Exception("Já existe uma composição cadastrada pra esse produto");
        }
        
        DB::transaction(function () use ($values, $final_product, $relations, $operation) {

            foreach ($values['composicao'] as $composicao) {

                $fill = array_merge($composicao, [
                    "pcp_fk_pve_codigo_acabado" => $values['pcp_fk_pve_codigo_acabado'],
                    "created_at" => Carbon::now()->format('Y-m-d H:i:s'),
                    "created_by" => auth()->user()->usu_nome
                ]);

                $composite_product = $this->repository->storeOrUpdate($fill, null, $relations)
                    ->load(['product_variety_pack', 'product_variety_pack.measurement_unit']);
                
                $this->composite_product_historic_repository->store(
                    $composite_product, 
                    $final_product,
                    $operation
                );
            }
        });

        return $this->product_variety_pack_repository->firstOrFail(["pve_codigo" => $values['pcp_fk_pve_codigo_acabado']],
            ['composition', 'product_variety']);
    }

    public function store($values, array $relations = []) {
        return $this->save($values, $relations, CompositeProductHistoricRepository::STORE);
    }

    public function update(array $values, int $id, array $relations = []) {
        $productionOrdersOpen = $this->production_order_repository->findManyOpen([
            'op_fk_pve_final' => $id
        ]);

        if (!$productionOrdersOpen->isEmpty()) {
            throw new \Exception("A composição selecionada possui ordens de produção em aberto vinculadas por isso não pode ser editada");
        }

        $compositions = $this->repository->findMany([
            'pcp_fk_pve_codigo_acabado' => $id
        ]);

        if ($compositions->isEmpty()) {
            throw new \Exception("O produto selecionado não possui nenhuma composição");
        }

        DB::transaction(function () use ($compositions) {
            foreach ($compositions as $composition) {
                $this->repository->destroy($composition->pcp_codigo);
            }
        });

        return $this->save($values, $relations, CompositeProductHistoricRepository::UPDATE);
    }

    public function showHistoric($id) {
        return $this->composite_product_historic_repository
            ->setOrder('created_at', 'desc')
            ->findMany(['pch_fk_pve_codigo_acabado' => $id]);
    }

    public function destroy(int $id)
    {

        return DB::transaction(function () use ($id) {

            $compositions = $this->repository->findMany(["pcp_fk_pve_codigo_acabado" => $id]);


            foreach ($compositions as $composition) {
                $this->repository->destroy($composition->pcp_codigo);
            }

            return $this->product_variety_pack_repository->firstOrFail(["pve_codigo" => $id],
                ['composition', 'product_variety'])
                ->load(['composition', 'product_variety']);
        });


    }
}
