<?php

namespace Schaffen\ProductionOrder\ProductionOrder\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Schaffen\Core\Product\Models\ProductVarietyPack;
use Schaffen\ProductionOrder\ProductionOrder\Models\ProductionOrderItem;


class ProductionOrder extends Model
{
    //
    use SoftDeletes;

    protected $table = "ordem_producao.ordem_producao";
    protected $fillable = [
        "op_fk_pve_final",
        "op_dta_agendamento",
        "op_hora_agendamento",
        "op_quantidade",
        "op_observacao"
    ];
    protected $primaryKey = "op_codigo";
    
    const UPDATED_AT = 'created_at';
    const CREATED_AT = 'updated_at';
    const DELETED_AT = 'deleted_at';

    public function status_po() {
        return $this->hasOne(ProductionOrderStatus::class, 'ops_fk_op_codigo')
            ->with('status');
    }

    public function status() {
        return $this->belongsToMany(Status::class, 'ordem_producao.op_status', 'ops_fk_op_codigo', 'ops_fk_status');
    }

    public function finished_product() {
        return $this->hasOne(ProductVarietyPack::class, 'pve_codigo', 'op_fk_pve_final')
            ->with('product_variety');
    }

    public function items() {
        return $this->hasMany(ProductionOrderItem::class, 'opi_fk_op_codigo', 'op_codigo');
    }
}
