<?php

namespace Schaffen\ProductionOrder\ProductionOrder\Repositories;

use DateTime;
use Freelabois\LaravelQuickstart\Extendables\Repository;
use Schaffen\ProductionOrder\ProductionOrder\Models\CompositeProductHistoric;
use Schaffen\ProductionOrder\ProductionOrder\Models\CompositeProduct;
use Schaffen\ProductionOrder\ProductionOrder\Models\ProductionOrder;
use Schaffen\Core\Product\Models\ProductVarietyPack;
use Illuminate\Support\Carbon;

class CompositeProductHistoricRepository extends Repository 
{
    protected $model = CompositeProductHistoric::class;

    protected $searchableFields = [
        ["field" => "pch_fk_pve_codigo_acabado"],
    ];

    const STORE = 'CADASTRO_PCP';
    const UPDATE = 'EDICAO_PCP';
    const STORE_OP = 'CADASTRO_ORDEM_PRODUCAO';

    public function store(CompositeProduct $composite_product, ProductVarietyPack $final_product, $operation) {

        $description = implode('<br>', [
            "Produto acabado: {$final_product->pve_codigo} - {$final_product->product_complete_name}",
            "Matéria-prima: {$composite_product->product_variety_pack->pve_codigo} - {$composite_product->product_variety_pack->product_complete_name}", 
            "Unidade de medida: {$composite_product->product_variety_pack->measurement_unit->unm_descricao}", 
            "Quantidade: " . number_format($composite_product->pcp_quantidade, 2, ',', '.'),
            "Valor unitário: " . number_format($composite_product->pcp_valor_unitario, 2, ',', '.'), 
            $composite_product->pcp_controla_estoque ? "Controla estoque: Sim" : "Controla estoque: Não",
            "Custo de produção previsto:" . number_format($composite_product->pcp_total, 2, ',', '.')
        ]);

        $values = [
            'pch_fk_pve_codigo_acabado' => $composite_product->pcp_fk_pve_codigo_acabado,
            'pch_operacao' => $operation,
            'pch_descricao' => $description,
            "created_at" => Carbon::now()->format('Y-m-d H:i:s'),
            "created_by" => auth()->user()->usu_nome
        ];

        $this->storeOrUpdate($values);
    }

    public function storeOP(ProductionOrder $production_order, $composition_products, $operation) {

        $date = new DateTime("{$production_order->op_data_agendamento} {$production_order->op_hora_agendamento}");

        $items = array_reduce($composition_products, function($text, $product) {
            $item = "{$product->feedstock->pve_codigo} - {$product->feedstock->product_complete_name}<br>";
            return $text . $item;
        }, 'Itens:<br>');

        $description = implode('<br>', [
            "Ordem de produção: {$production_order->op_codigo}", 
            "Produto acabado: {$production_order->finished_product->pve_codigo} – {$production_order->finished_product->product_complete_name}", 
            "Data de agendamento: " . $date->format('d/m/Y'), 
            "Hora de agendamento: " . $date->format('H:i:s'), 
            "Quantidade prevista: {$production_order->op_quantidade}", 
            "Observação: {$production_order->op_observacao}",
            $items
        ]);

        $values = [
            'pch_fk_pve_codigo_acabado' => $production_order->finished_product->pve_codigo,
            'pch_operacao' => $operation,
            'pch_descricao' => $description,
            "created_at" => Carbon::now()->format('Y-m-d H:i:s'),
            "created_by" => auth()->user()->usu_nome
        ];

        $this->storeOrUpdate($values);
    }


}